package xylia.sax.contentmodelparser;

/** An interface for objects that create content models.
 * This interface is intended for use with the ContentModelParser.
 * 
 * Copyright:    Copyright (c) 2001
 * Company:      Memorial
 * @author Theodore Norvell
 * @version 1.0
 */

public interface ContentModelFactory extends ContentModelKinds {

    /** Complete construction of a content model.
     *  @param kind, either ANY or EMPTY
     */
    Object finishContentModel( int kind ) ;

    /** Complete construction of a content model.
     *  @param kind Either MIXED or CHILDREN.
     *  @param regExp An object representing the regular expression.
     *  @returns An object representing the parsed content model.
     */
    Object finishContentModel(  int kind, Object regExp ) ;

    /** Create a regular expression object representing an occurence of #PCDATA in the content model.
     *  @returns An object representing the regular expression.
     */
    Object mkPCDATA( ) ;

    /** Create a regular expression object representing an occurence of an element name in the content model.
     *  @param name The element name.
     *  @returns An object representing the regular expression.
     */
    Object mkName( String name ) ;

    /** Create a regular expression object representing an occurence of an alternation in the content model.
     *  An alternation is a choice of two regular expressions: left | right
     *  @param left A regular expression object representing the left operand.
     *  @param right A regular expression object representing the right operand.
     *  @returns An object representing the regular expression.
     */
    Object mkAlternation( Object right, Object left ) ;

    /** Create a regular expression object representing an occurence of a sequence in the content model.
     *  A sequence is a succession of two regular expressions: left , right
     *  @param left A regular expression object representing the left operand.
     *  @param right A regular expression object representing the right operand.
     *  @returns An object representing the regular expression.
     */
    Object mkSequence( Object right, Object left ) ;

    /** Create a regular expression object representing an occurence of optional regular expression in the content model.
     *  An optional regular expression represents something that may or may not
     *  be there: operand?
     *  @param operand A regular expression object representing the operand.
     *  @returns An object representing the regular expression.
     */
    Object mkOptional( Object operand ) ;

    /** Create a regular expression object representing possibly empty repetition of a regular expression in the content model.
     *  A possibly empty repetition of a regular expression represents something that may be repeated
     *  any number of times: operand*
     *  @param operand A regular expression object representing the operand.
     *  @returns An object representing the regular expression.
     */
    Object mkKleeneStar( Object operand ) ;

    /** Create a regular expression object representing nonempty repetition of a regular expression in the content model.
     *  A nonempty repetition of a regular expression represents something that may be repeated
     *  any number of times: operand+
     *  @param operand A regular expression object representing the operand.
     *  @returns An object representing the regular expression.
     */
    Object mkKleenePlus( Object operand ) ;
}