package xylia.sax.contentmodelparser;

import java.util.*;

/** A Finite State Automaton.
 *  <p>An FSA consists of a start state, a state transition structure and
 *  a set of final states.</p>
 * <p>Copyright: Copyright (c) 2002 Theodore S. Norvell</p>
 * <p>Company: MUN</p>
 * @author Theodore S. Norvell
 * @version 1.0
 */

public class FiniteStateAutomaton {

    private StateTransitionStructure delta ;
    private Object startState ;
    private Set finalStates ;
    
    public FiniteStateAutomaton(StateTransitionStructure d, Object s, Set f) {
        delta = d ;
        startState = s ;
        finalStates = f ; }
    
    /** Returns true iff the List is matched by the automaton.
     *  
     */
    public boolean match( List l ) {
        Set startSet = new HashSet() ;
        startSet.add( startState ) ;
        Set endSet = delta.goForwardOverList(startSet, l ) ;
        Set intersection = new HashSet() ;
        intersection.addAll( endSet ) ; intersection.retainAll( finalStates ) ;
        return ! intersection.isEmpty() ; }
        
    public String toString() {
        return "FSA { start = "+startState+"\n"
             + "      final = "+finalStates+"\n"
             + "      delta = "+delta+"\n}" ;
    }
}