package xylia.sax.contentmodelparser;

import java.util.* ;

import xylia.utilities.Assert ;

/** A graph structure for Finite State Automata. The FSA need not be deterimistic.
 * <p>Title: Xylia</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2001</p>
 * <p>Company: MUN</p>
 * @author Theodore S. Norvell
 * @version 1.0
 */

public class StateTransitionStructure {

    private Map forward ;
    private Map backward ;
    
    public StateTransitionStructure() {
        forward = new HashMap() ;
        backward = new HashMap() ;
    }
    
    public void addState( Object s ) {
        if( ! forward.containsKey( s ) ) {
            forward.put( s, new HashMap() ) ;
            backward.put( s, new HashMap() ) ; }
    }
    
    public void addTransition( Object startState, Object label, Object endState ) {
        Assert.assert( forward.containsKey( startState ) ) ;
        Assert.assert( forward.containsKey( endState ) ) ;
        
        // Add endState to forward(startState)(label)
        Map forwardStateMap = (Map) forward.get( startState ) ;
        Set destinationSet ;
        if( forwardStateMap.containsKey( label ) ) {
            destinationSet = (Set) forwardStateMap.get( label ) ; }
        else {
            destinationSet = new HashSet() ;
            forwardStateMap.put( label, destinationSet ) ; }
        destinationSet.add( endState ) ;
        
        // Add startState to backward(endState)(label)
        Map backwardStateMap = (Map) backward.get( endState ) ;
        Set sourceSet ;
        if( backwardStateMap.containsKey( label ) ) {
            sourceSet = (Set) backwardStateMap.get( label ) ; }
        else {
            sourceSet = new HashSet() ;
            backwardStateMap.put( label, sourceSet ) ; }
        sourceSet.add( startState ) ; }
    
    /** Obtain the set of states that follows a given state. */
    public Set goForward( Object startState, Object label ) {
        Assert.assert( forward.containsKey( startState ) ) ;
        Map forwardStateMap = (Map) forward.get( startState ) ;
        if( forwardStateMap.containsKey( label ) ) {
            return (Set) forwardStateMap.get( label ) ; }
        else {
            return new HashSet() ; } }
    
    /** Obtain the set of states that preceed a given state. */
    public Set goBackward( Object endState, Object label ) {
        Assert.assert( backward.containsKey( endState ) ) ;
        Map backwardStateMap = (Map) backward.get( endState ) ;
        if( backwardStateMap.containsKey( label ) ) {
            return (Set) backwardStateMap.get( label ) ; }
        else {
            return new HashSet() ; } }
    
    /** Obtain the set of states reachable from any state in a set, by a list of labels. */
    public Set goForwardOverList( Set startStates, List labels ) {
        Set current = startStates ;
        Iterator i = labels.listIterator() ;
        while( i.hasNext() && ! current.isEmpty() ) {
            Object label = i.next() ;
            Set nextStates = new HashSet() ;
            Iterator j = current.iterator() ;
            while( j.hasNext() ) {
                Object state = j.next() ; 
                Set nextForState = goForward( state, label ) ;
                nextStates.addAll( nextForState ) ; }
            current = nextStates ; }
        return current ; }
        
    /** Obtain the set of states that reaches any state in a set, by a list of labels. */
    public Set goBackwardOverList( Set startStates, List labels ) {
        Set current = startStates ;
        ListIterator i = labels.listIterator() ;
        while( i.hasPrevious() && ! current.isEmpty() ) {
            Object label = i.previous() ;
            Set nextStates = new HashSet() ;
            Iterator j = current.iterator() ;
            while( j.hasNext() ) {
                Object state = j.next() ; 
                Set previousForState = goBackward( state, label ) ;
                nextStates.addAll( previousForState ) ; }
            current = nextStates ; }
        return current ; }

    public String toString() {
        return "STS { forward = "+forward+"\n"
              +"      backward= "+backward+"\n}" ; }        
}